@@ -8,11 +8,8 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkRectOval.c,v 1.17 2007/12/13 15:24:16 dgp Exp $
  */
 
-#include <stdio.h>
 #include "tkInt.h"
 #include "tkCanvas.h"
 
@@ -45,40 +42,35 @@ typedef struct RectOvalItem  {
  * Information used for parsing configuration specs:
  */
 
-static Tk_CustomOption stateOption = {
-    (Tk_OptionParseProc *) TkStateParseProc,
-    TkStatePrintProc, (ClientData) 2
+static const Tk_CustomOption stateOption = {
+    TkStateParseProc, TkStatePrintProc, INT2PTR(2)
 };
-static Tk_CustomOption tagsOption = {
-    (Tk_OptionParseProc *) Tk_CanvasTagsParseProc,
-    Tk_CanvasTagsPrintProc, (ClientData) NULL
+static const Tk_CustomOption tagsOption = {
+    Tk_CanvasTagsParseProc, Tk_CanvasTagsPrintProc, NULL
 };
-static Tk_CustomOption dashOption = {
-    (Tk_OptionParseProc *) TkCanvasDashParseProc,
-    TkCanvasDashPrintProc, (ClientData) NULL
+static const Tk_CustomOption dashOption = {
+    TkCanvasDashParseProc, TkCanvasDashPrintProc, NULL
 };
-static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) TkOffsetParseProc,
-    TkOffsetPrintProc, (ClientData) TK_OFFSET_RELATIVE
+static const Tk_CustomOption offsetOption = {
+    TkOffsetParseProc, TkOffsetPrintProc, INT2PTR(TK_OFFSET_RELATIVE)
 };
-static Tk_CustomOption pixelOption = {
-    (Tk_OptionParseProc *) TkPixelParseProc,
-    TkPixelPrintProc, (ClientData) NULL
+static const Tk_CustomOption pixelOption = {
+    TkPixelParseProc, TkPixelPrintProc, NULL
 };
 
-static Tk_ConfigSpec configSpecs[] = {
+static const Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_CUSTOM, "-activedash", NULL, NULL,
 	NULL, Tk_Offset(RectOvalItem, outline.activeDash),
 	TK_CONFIG_NULL_OK, &dashOption},
     {TK_CONFIG_COLOR, "-activefill", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, activeFillColor), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, activeFillColor), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_COLOR, "-activeoutline", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, outline.activeColor), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, outline.activeColor), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_BITMAP, "-activeoutlinestipple", NULL, NULL,
 	NULL, Tk_Offset(RectOvalItem, outline.activeStipple),
-	TK_CONFIG_NULL_OK},
+	TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_BITMAP, "-activestipple", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, activeFillStipple), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, activeFillStipple), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_CUSTOM, "-activewidth", NULL, NULL,
 	"0.0", Tk_Offset(RectOvalItem, outline.activeWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
@@ -87,45 +79,45 @@ static Tk_ConfigSpec configSpecs[] = {
 	TK_CONFIG_NULL_OK, &dashOption},
     {TK_CONFIG_PIXELS, "-dashoffset", NULL, NULL,
 	"0", Tk_Offset(RectOvalItem, outline.offset),
-	TK_CONFIG_DONT_SET_DEFAULT},
+	TK_CONFIG_DONT_SET_DEFAULT, NULL},
     {TK_CONFIG_CUSTOM, "-disableddash", NULL, NULL,
 	NULL, Tk_Offset(RectOvalItem, outline.disabledDash),
 	TK_CONFIG_NULL_OK, &dashOption},
     {TK_CONFIG_COLOR, "-disabledfill", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, disabledFillColor), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, disabledFillColor), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_COLOR, "-disabledoutline", NULL, NULL,
 	NULL, Tk_Offset(RectOvalItem, outline.disabledColor),
-	TK_CONFIG_NULL_OK},
+	TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_BITMAP, "-disabledoutlinestipple", NULL, NULL,
 	NULL, Tk_Offset(RectOvalItem, outline.disabledStipple),
-	TK_CONFIG_NULL_OK},
+	TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_BITMAP, "-disabledstipple", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, disabledFillStipple), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, disabledFillStipple), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_PIXELS, "-disabledwidth", NULL, NULL,
 	"0.0", Tk_Offset(RectOvalItem, outline.disabledWidth),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
     {TK_CONFIG_COLOR, "-fill", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, fillColor), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, fillColor), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_CUSTOM, "-offset", NULL, NULL,
 	"0,0", Tk_Offset(RectOvalItem, tsoffset),
 	TK_CONFIG_DONT_SET_DEFAULT, &offsetOption},
     {TK_CONFIG_COLOR, "-outline", NULL, NULL,
-	"black", Tk_Offset(RectOvalItem, outline.color), TK_CONFIG_NULL_OK},
+	"black", Tk_Offset(RectOvalItem, outline.color), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_CUSTOM, "-outlineoffset", NULL, NULL,
 	"0,0", Tk_Offset(RectOvalItem, outline.tsoffset),
 	TK_CONFIG_DONT_SET_DEFAULT, &offsetOption},
     {TK_CONFIG_BITMAP, "-outlinestipple", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, outline.stipple), TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, outline.stipple), TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_CUSTOM, "-state", NULL, NULL,
 	NULL, Tk_Offset(Tk_Item, state),TK_CONFIG_NULL_OK, &stateOption},
     {TK_CONFIG_BITMAP, "-stipple", NULL, NULL,
-	NULL, Tk_Offset(RectOvalItem, fillStipple),TK_CONFIG_NULL_OK},
+	NULL, Tk_Offset(RectOvalItem, fillStipple),TK_CONFIG_NULL_OK, NULL},
     {TK_CONFIG_CUSTOM, "-tags", NULL, NULL,
 	NULL, 0, TK_CONFIG_NULL_OK, &tagsOption},
     {TK_CONFIG_CUSTOM, "-width", NULL, NULL,
 	"1.0", Tk_Offset(RectOvalItem, outline.width),
 	TK_CONFIG_DONT_SET_DEFAULT, &pixelOption},
-    {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
+    {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0, NULL}
 };
 
 /*
@@ -135,10 +127,10 @@ static Tk_ConfigSpec configSpecs[] = {
 static void		ComputeRectOvalBbox(Tk_Canvas canvas,
 			    RectOvalItem *rectOvalPtr);
 static int		ConfigureRectOval(Tcl_Interp *interp, Tk_Canvas canvas,
-			    Tk_Item *itemPtr, int objc, Tcl_Obj *CONST objv[],
+			    Tk_Item *itemPtr, int objc, Tcl_Obj *const objv[],
 			    int flags);
 static int		CreateRectOval(Tcl_Interp *interp, Tk_Canvas canvas,
-			    Tk_Item *itemPtr, int objc, Tcl_Obj *CONST objv[]);
+			    Tk_Item *itemPtr, int objc, Tcl_Obj *const objv[]);
 static void		DeleteRectOval(Tk_Canvas canvas, Tk_Item *itemPtr,
 			    Display *display);
 static void		DisplayRectOval(Tk_Canvas canvas, Tk_Item *itemPtr,
@@ -149,7 +141,7 @@ static int		OvalToArea(Tk_Canvas canvas, Tk_Item *itemPtr,
 static double		OvalToPoint(Tk_Canvas canvas, Tk_Item *itemPtr,
 			    double *pointPtr);
 static int		RectOvalCoords(Tcl_Interp *interp, Tk_Canvas canvas,
-			    Tk_Item *itemPtr, int objc, Tcl_Obj *CONST objv[]);
+			    Tk_Item *itemPtr, int objc, Tcl_Obj *const objv[]);
 static int		RectOvalToPostscript(Tcl_Interp *interp,
 			    Tk_Canvas canvas, Tk_Item *itemPtr, int prepass);
 static int		RectToArea(Tk_Canvas canvas, Tk_Item *itemPtr,
@@ -188,6 +180,7 @@ Tk_ItemType tkRectangleType = {
     NULL,			/* insertProc */
     NULL,			/* dTextProc */
     NULL,			/* nextPtr */
+    NULL, 0, NULL, NULL
 };
 
 Tk_ItemType tkOvalType = {
@@ -211,6 +204,7 @@ Tk_ItemType tkOvalType = {
     NULL,			/* insertProc */
     NULL,			/* dTextProc */
     NULL,			/* nextPtr */
+    NULL, 0, NULL, NULL
 };
 
 /*
@@ -240,13 +234,13 @@ CreateRectOval(
     Tk_Item *itemPtr,		/* Record to hold new item; header has been
 				 * initialized by caller. */
     int objc,			/* Number of arguments in objv. */
-    Tcl_Obj *CONST objv[])	/* Arguments describing rectangle. */
+    Tcl_Obj *const objv[])	/* Arguments describing rectangle. */
 {
     RectOvalItem *rectOvalPtr = (RectOvalItem *) itemPtr;
     int i;
 
     if (objc == 0) {
-	Tcl_Panic("canvas did not pass any coords\n");
+	Tcl_Panic("canvas did not pass any coords");
     }
 
     /*
@@ -271,7 +265,7 @@ CreateRectOval(
      */
 
     for (i = 1; i < objc; i++) {
-	char *arg = Tcl_GetString(objv[i]);
+	const char *arg = Tcl_GetString(objv[i]);
 
 	if ((arg[0] == '-') && (arg[1] >= 'a') && (arg[1] <= 'z')) {
 	    break;
@@ -315,7 +309,7 @@ RectOvalCoords(
     Tk_Item *itemPtr,		/* Item whose coordinates are to be read or
 				 * modified. */
     int objc,			/* Number of coordinates supplied in objv. */
-    Tcl_Obj *CONST objv[])	/* Array of coordinates: x1,y1,x2,y2,... */
+    Tcl_Obj *const objv[])	/* Array of coordinates: x1,y1,x2,y2,... */
 {
     RectOvalItem *rectOvalPtr = (RectOvalItem *) itemPtr;
 
@@ -324,17 +318,13 @@ RectOvalCoords(
      */
 
     if (objc == 0) {
-	Tcl_Obj *obj = Tcl_NewObj();
+	Tcl_Obj *bbox[4];
 
-	Tcl_ListObjAppendElement(NULL, obj,
-		Tcl_NewDoubleObj(rectOvalPtr->bbox[0]));
-	Tcl_ListObjAppendElement(NULL, obj,
-		Tcl_NewDoubleObj(rectOvalPtr->bbox[1]));
-	Tcl_ListObjAppendElement(NULL, obj,
-		Tcl_NewDoubleObj(rectOvalPtr->bbox[2]));
-	Tcl_ListObjAppendElement(NULL, obj,
-		Tcl_NewDoubleObj(rectOvalPtr->bbox[3]));
-	Tcl_SetObjResult(interp, obj);
+	bbox[0] = Tcl_NewDoubleObj(rectOvalPtr->bbox[0]);
+	bbox[1] = Tcl_NewDoubleObj(rectOvalPtr->bbox[1]);
+	bbox[2] = Tcl_NewDoubleObj(rectOvalPtr->bbox[2]);
+	bbox[3] = Tcl_NewDoubleObj(rectOvalPtr->bbox[3]);
+	Tcl_SetObjResult(interp, Tcl_NewListObj(4, bbox));
 	return TCL_OK;
     }
 
@@ -354,10 +344,11 @@ RectOvalCoords(
      */
 
     if (objc != 4) {
-	char buf[64 + TCL_INTEGER_SPACE];
-
-	sprintf(buf, "wrong # coordinates: expected 0 or 4, got %d", objc);
-	Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"wrong # coordinates: expected 0 or 4, got %d", objc));
+	Tcl_SetErrorCode(interp, "TK", "CANVAS", "COORDS",
+		(rectOvalPtr->header.typePtr == &tkRectangleType
+			? "RECTANGLE" : "OVAL"), NULL);
 	return TCL_ERROR;
     }
 
@@ -404,7 +395,7 @@ ConfigureRectOval(
     Tk_Canvas canvas,		/* Canvas containing itemPtr. */
     Tk_Item *itemPtr,		/* Rectangle item to reconfigure. */
     int objc,			/* Number of elements in objv. */
-    Tcl_Obj *CONST objv[],	/* Arguments describing things to configure. */
+    Tcl_Obj *const objv[],	/* Arguments describing things to configure. */
     int flags)			/* Flags to pass to Tk_ConfigureWidget. */
 {
     RectOvalItem *rectOvalPtr = (RectOvalItem *) itemPtr;
@@ -420,7 +411,7 @@ ConfigureRectOval(
     tkwin = Tk_CanvasTkwin(canvas);
 
     if (TCL_OK != Tk_ConfigureWidget(interp, tkwin, configSpecs, objc,
-	    (CONST char **)objv, (char *) rectOvalPtr, flags|TK_CONFIG_OBJS)) {
+	    (const char **)objv, (char *) rectOvalPtr, flags|TK_CONFIG_OBJS)) {
 	return TCL_ERROR;
     }
     state = itemPtr->state;
@@ -483,7 +474,7 @@ ConfigureRectOval(
     rectOvalPtr->outline.gc = newGC;
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
     if (state == TK_STATE_HIDDEN) {
 	ComputeRectOvalBbox(canvas, rectOvalPtr);
@@ -492,7 +483,7 @@ ConfigureRectOval(
 
     color = rectOvalPtr->fillColor;
     stipple = rectOvalPtr->fillStipple;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
+    if (Canvas(canvas)->currentItemPtr == itemPtr) {
 	if (rectOvalPtr->activeFillColor!=NULL) {
 	    color = rectOvalPtr->activeFillColor;
 	}
@@ -521,9 +512,10 @@ ConfigureRectOval(
 	}
 #ifdef MAC_OSX_TK
 	/*
-	 * Mac OS X CG drawing needs access to the outline linewidth
-	 * even for fills (as linewidth controls antialiasing).
+	 * Mac OS X CG drawing needs access to the outline linewidth even for
+	 * fills (as linewidth controls antialiasing).
 	 */
+
 	gcValues.line_width = rectOvalPtr->outline.gc != None ?
 		rectOvalPtr->outline.gc->line_width : 0;
 	mask |= GCLineWidth;
@@ -636,7 +628,7 @@ ComputeRectOvalBbox(
     Tk_State state = rectOvalPtr->header.state;
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
 
     width = rectOvalPtr->outline.width;
@@ -645,7 +637,7 @@ ComputeRectOvalBbox(
 	rectOvalPtr->header.x2 = rectOvalPtr->header.y2 = -1;
 	return;
     }
-    if (((TkCanvas *)canvas)->currentItemPtr == (Tk_Item *)rectOvalPtr) {
+    if (Canvas(canvas)->currentItemPtr == (Tk_Item *) rectOvalPtr) {
 	if (rectOvalPtr->outline.activeWidth>width) {
 	    width = rectOvalPtr->outline.activeWidth;
 	}
@@ -679,11 +671,11 @@ ComputeRectOvalBbox(
 	 * correct place to solve it, but it works.
 	 */
 
-#ifdef __WIN32__
+#ifdef _WIN32
 	bloat = 1;
 #else
 	bloat = 0;
-#endif
+#endif /* _WIN32 */
     } else {
 #ifdef MAC_OSX_TK
 	/*
@@ -695,7 +687,7 @@ ComputeRectOvalBbox(
 	bloat = (int) (width+1.5)/2;
 #else
 	bloat = (int) (width+1)/2;
-#endif
+#endif /* MAC_OSX_TK */
     }
 
     /*
@@ -763,15 +755,107 @@ DisplayRectOval(
      * will die if it isn't.
      */
 
-    Tk_CanvasDrawableCoords(canvas, rectOvalPtr->bbox[0], rectOvalPtr->bbox[1],
+    Tk_CanvasDrawableCoords(canvas, rectOvalPtr->bbox[0],rectOvalPtr->bbox[1],
 	    &x1, &y1);
-    Tk_CanvasDrawableCoords(canvas, rectOvalPtr->bbox[2], rectOvalPtr->bbox[3],
+    Tk_CanvasDrawableCoords(canvas, rectOvalPtr->bbox[2],rectOvalPtr->bbox[3],
 	    &x2, &y2);
-    if (x2 <= x1) {
-	x2 = x1+1;
+    if (x2 == x1) {
+
+        /*
+         * The width of the bounding box corresponds to less than one pixel
+         * on screen. Adjustment is needed to avoid drawing attempts with zero
+         * width items (which would draw nothing). The bounding box spans
+         * either 1 or 2 pixels. Select which pixel will be drawn.
+         */
+
+        short ix1 = (short) (rectOvalPtr->bbox[0]);
+        short ix2 = (short) (rectOvalPtr->bbox[2]);
+
+        if (ix1 == ix2) {
+
+            /*
+             * x1 and x2 are "within the same pixel". Use this pixel.
+             * Note: the degenerated case (bbox[0]==bbox[2]) of a completely
+             * flat box results in arbitrary selection of the pixel at the
+             * right (with positive coordinate) or left (with negative
+             * coordinate) of the box. There is no "best choice" here.
+             */
+
+            if (ix1 > 0) {
+                x2 += 1;
+            } else {
+                x1 -= 1;
+            }
+        } else {
+
+            /*
+             * (x1,x2) span two pixels. Select the one with the larger
+             * covered "area".
+             */
+
+            if (ix1 > 0) {
+                if ((rectOvalPtr->bbox[2] - ix2) > (ix2 - rectOvalPtr->bbox[0])) {
+                    x2 += 1;
+                } else {
+                    x1 -= 1;
+                }
+            } else {
+                if ((rectOvalPtr->bbox[2] - ix1) > (ix1 - rectOvalPtr->bbox[0])) {
+                    x2 += 1;
+                } else {
+                    x1 -= 1;
+                }
+            }
+        }
     }
-    if (y2 <= y1) {
-	y2 = y1+1;
+    if (y2 == y1) {
+
+        /*
+         * The height of the bounding box corresponds to less than one pixel
+         * on screen. Adjustment is needed to avoid drawing attempts with zero
+         * height items (which would draw nothing). The bounding box spans
+         * either 1 or 2 pixels. Select which pixel will be drawn.
+         */
+
+        short iy1 = (short) (rectOvalPtr->bbox[1]);
+        short iy2 = (short) (rectOvalPtr->bbox[3]);
+
+        if (iy1 == iy2) {
+
+            /*
+             * y1 and y2 are "within the same pixel". Use this pixel.
+             * Note: the degenerated case (bbox[1]==bbox[3]) of a completely
+             * flat box results in arbitrary selection of the pixel below
+             * (with positive coordinate) or above (with negative coordinate)
+             * the box. There is no "best choice" here.
+             */
+
+            if (iy1 > 0) {
+                y2 += 1;
+            } else {
+                y1 -= 1;
+            }
+        } else {
+
+            /*
+             * (y1,y2) span two pixels. Select the one with the larger
+             * covered "area".
+             */
+
+            if (iy1 > 0) {
+                if ((rectOvalPtr->bbox[3] - iy2) > (iy2 - rectOvalPtr->bbox[1])) {
+                    y2 += 1;
+                } else {
+                    y1 -= 1;
+                }
+            } else {
+                if ((rectOvalPtr->bbox[3] - iy1) > (iy1 - rectOvalPtr->bbox[1])) {
+                    y2 += 1;
+                } else {
+                    y1 -= 1;
+                }
+            }
+        }
     }
 
     /*
@@ -781,10 +865,10 @@ DisplayRectOval(
      */
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
     fillStipple = rectOvalPtr->fillStipple;
-    if (((TkCanvas *)canvas)->currentItemPtr == (Tk_Item *)rectOvalPtr) {
+    if (Canvas(canvas)->currentItemPtr == (Tk_Item *) rectOvalPtr) {
 	if (rectOvalPtr->activeFillStipple != None) {
 	    fillStipple = rectOvalPtr->activeFillStipple;
 	}
@@ -886,11 +970,11 @@ RectToPoint(
     Tk_State state = itemPtr->state;
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
 
     width = rectPtr->outline.width;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
+    if (Canvas(canvas)->currentItemPtr == itemPtr) {
 	if (rectPtr->outline.activeWidth>width) {
 	    width = rectPtr->outline.activeWidth;
 	}
@@ -1006,11 +1090,11 @@ OvalToPoint(
     Tk_State state = itemPtr->state;
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
 
     width = (double) ovalPtr->outline.width;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
+    if (Canvas(canvas)->currentItemPtr == itemPtr) {
 	if (ovalPtr->outline.activeWidth>width) {
 	    width = (double) ovalPtr->outline.activeWidth;
 	}
@@ -1062,16 +1146,16 @@ RectToArea(
     Tk_State state = itemPtr->state;
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
 
     width = rectPtr->outline.width;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (rectPtr->outline.activeWidth>width) {
+    if (Canvas(canvas)->currentItemPtr == itemPtr) {
+	if (rectPtr->outline.activeWidth > width) {
 	    width = rectPtr->outline.activeWidth;
 	}
     } else if (state == TK_STATE_DISABLED) {
-	if (rectPtr->outline.disabledWidth>0) {
+	if (rectPtr->outline.disabledWidth > 0) {
 	    width = rectPtr->outline.disabledWidth;
 	}
     }
@@ -1131,22 +1215,21 @@ OvalToArea(
 				 * y1, x2, y2) describing rectangular area. */
 {
     RectOvalItem *ovalPtr = (RectOvalItem *) itemPtr;
-    double oval[4], halfWidth;
+    double oval[4], halfWidth, width;
     int result;
-    double width;
     Tk_State state = itemPtr->state;
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
 
     width = ovalPtr->outline.width;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
-	if (ovalPtr->outline.activeWidth>width) {
+    if (Canvas(canvas)->currentItemPtr == itemPtr) {
+	if (ovalPtr->outline.activeWidth > width) {
 	    width = ovalPtr->outline.activeWidth;
 	}
     } else if (state == TK_STATE_DISABLED) {
-	if (ovalPtr->outline.disabledWidth>0) {
+	if (ovalPtr->outline.disabledWidth > 0) {
 	    width = ovalPtr->outline.disabledWidth;
 	}
     }
@@ -1300,13 +1383,14 @@ RectOvalToPostscript(
 				 * information; 0 means final Postscript is
 				 * being created. */
 {
-    char pathCmd[500];
+    Tcl_Obj *pathObj, *psObj;
     RectOvalItem *rectOvalPtr = (RectOvalItem *) itemPtr;
     double y1, y2;
     XColor *color;
     XColor *fillColor;
     Pixmap fillStipple;
     Tk_State state = itemPtr->state;
+    Tcl_InterpState interpState;
 
     y1 = Tk_CanvasPsY(canvas, rectOvalPtr->bbox[1]);
     y2 = Tk_CanvasPsY(canvas, rectOvalPtr->bbox[3]);
@@ -1317,23 +1401,34 @@ RectOvalToPostscript(
      */
 
     if (rectOvalPtr->header.typePtr == &tkRectangleType) {
-	sprintf(pathCmd, "%.15g %.15g moveto %.15g 0 rlineto 0 %.15g rlineto %.15g 0 rlineto closepath\n",
+	pathObj = Tcl_ObjPrintf(
+		"%.15g %.15g moveto "
+		"%.15g 0 rlineto "
+		"0 %.15g rlineto "
+		"%.15g 0 rlineto "
+		"closepath\n",
 		rectOvalPtr->bbox[0], y1,
-		rectOvalPtr->bbox[2]-rectOvalPtr->bbox[0], y2-y1,
+		rectOvalPtr->bbox[2]-rectOvalPtr->bbox[0],
+		y2-y1,
 		rectOvalPtr->bbox[0]-rectOvalPtr->bbox[2]);
     } else {
-	sprintf(pathCmd, "matrix currentmatrix\n%.15g %.15g translate %.15g %.15g scale 1 0 moveto 0 0 1 0 360 arc\nsetmatrix\n",
+	pathObj = Tcl_ObjPrintf(
+		"matrix currentmatrix\n"
+		"%.15g %.15g translate "
+		"%.15g %.15g scale "
+		"1 0 moveto 0 0 1 0 360 arc\n"
+		"setmatrix\n",
 		(rectOvalPtr->bbox[0] + rectOvalPtr->bbox[2])/2, (y1 + y2)/2,
 		(rectOvalPtr->bbox[2] - rectOvalPtr->bbox[0])/2, (y1 - y2)/2);
     }
 
     if (state == TK_STATE_NULL) {
-	state = ((TkCanvas *)canvas)->canvas_state;
+	state = Canvas(canvas)->canvas_state;
     }
     color = rectOvalPtr->outline.color;
     fillColor = rectOvalPtr->fillColor;
     fillStipple = rectOvalPtr->fillStipple;
-    if (((TkCanvas *)canvas)->currentItemPtr == itemPtr) {
+    if (Canvas(canvas)->currentItemPtr == itemPtr) {
 	if (rectOvalPtr->outline.activeColor!=NULL) {
 	    color = rectOvalPtr->outline.activeColor;
 	}
@@ -1355,25 +1450,39 @@ RectOvalToPostscript(
 	}
     }
 
+    /*
+     * Make our working space.
+     */
+
+    psObj = Tcl_NewObj();
+    interpState = Tcl_SaveInterpState(interp, TCL_OK);
+
     /*
      * First draw the filled area of the rectangle.
      */
 
     if (fillColor != NULL) {
-	Tcl_AppendResult(interp, pathCmd, NULL);
+	Tcl_AppendObjToObj(psObj, pathObj);
+
+	Tcl_ResetResult(interp);
 	if (Tk_CanvasPsColor(interp, canvas, fillColor) != TCL_OK) {
-	    return TCL_ERROR;
+	    goto error;
 	}
+	Tcl_AppendObjToObj(psObj, Tcl_GetObjResult(interp));
+
 	if (fillStipple != None) {
-	    Tcl_AppendResult(interp, "clip ", NULL);
+	    Tcl_AppendToObj(psObj, "clip ", -1);
+
+	    Tcl_ResetResult(interp);
 	    if (Tk_CanvasPsStipple(interp, canvas, fillStipple) != TCL_OK) {
-		return TCL_ERROR;
+		goto error;
 	    }
+	    Tcl_AppendObjToObj(psObj, Tcl_GetObjResult(interp));
 	    if (color != NULL) {
-		Tcl_AppendResult(interp, "grestore gsave\n", NULL);
+		Tcl_AppendToObj(psObj, "grestore gsave\n", -1);
 	    }
 	} else {
-	    Tcl_AppendResult(interp, "fill\n", NULL);
+	    Tcl_AppendToObj(psObj, "fill\n", -1);
 	}
     }
 
@@ -1382,14 +1491,32 @@ RectOvalToPostscript(
      */
 
     if (color != NULL) {
-	Tcl_AppendResult(interp, pathCmd, "0 setlinejoin 2 setlinecap\n",
-		NULL);
+	Tcl_AppendObjToObj(psObj, pathObj);
+	Tcl_AppendToObj(psObj, "0 setlinejoin 2 setlinecap\n", -1);
+
+	Tcl_ResetResult(interp);
 	if (Tk_CanvasPsOutline(canvas, itemPtr,
-		&(rectOvalPtr->outline))!= TCL_OK) {
-	    return TCL_ERROR;
+		&rectOvalPtr->outline)!= TCL_OK) {
+	    goto error;
 	}
+	Tcl_AppendObjToObj(psObj, Tcl_GetObjResult(interp));
     }
+
+    /*
+     * Plug the accumulated postscript back into the result.
+     */
+
+    (void) Tcl_RestoreInterpState(interp, interpState);
+    Tcl_AppendObjToObj(Tcl_GetObjResult(interp), psObj);
+    Tcl_DecrRefCount(psObj);
+    Tcl_DecrRefCount(pathObj);
     return TCL_OK;
+
+  error:
+    Tcl_DiscardInterpState(interpState);
+    Tcl_DecrRefCount(psObj);
+    Tcl_DecrRefCount(pathObj);
+    return TCL_ERROR;
 }
 
 /*
